@@ -3,7 +3,6 @@ import requests
 from web3 import Web3
 from decimal import *
 import importlib.resources as pkg_resources
-from functools import reduce
 from web3.middleware import geth_poa_middleware
 
 
@@ -15,7 +14,8 @@ class OneInchSwap:
     base_url = 'https://api.1inch.exchange'
 
     version = {
-        "v4.0": "v4.0"
+        "v4.0": "v4.0",
+        "v5.0": "v5.0"
     }
 
     chains = {
@@ -29,7 +29,7 @@ class OneInchSwap:
         "fantom": "250"
     }
 
-    def __init__(self, address, chain='ethereum', version='v4.0'):
+    def __init__(self, address, chain='ethereum', version='v5.0'):
         self.presets = None
         self.tokens = {}
         self.tokens_by_address = {}
@@ -195,37 +195,43 @@ class OneInchSwap:
 
 
 class TransactionHelper:
-    gas_oracles = {
-        # "ethereum": 'https://etherchain.org/api/gasnow',
-        "binance": 'https://gbsc.blockscan.com/gasapi.ashx?apikey=key&method=gasoracle',
-        # "polygon": "https://gasstation-mainnet.matic.network/",
-        # "optimism": "10",
-        # "arbitrum": "https://owlracle.info/arb/gas?apikey=877d078a2753422d8a6fbd3d092f5ddb",
-        # "gnosis": "https://blockscout.com/xdai/mainnet/api/v1/gas-price-oracle",
-        # "avalanche": "https://gavax.blockscan.com/gasapi.ashx?apikey=key&method=gasoracle",
-        # "fantom": "https://owlracle.info/ftm/gas?apikey=877d078a2753422d8a6fbd3d092f5ddb"
-    }
+    gas_oracle = "https://gas-price-api.1inch.io/v1.3/"
 
     chains = {
         "ethereum": '1',
         "binance": '56',
         "polygon": "137",
-        # "optimism": "10",
-        # "arbitrum": "42161",
-        # "gnosis": "100",
+        "optimism": "10",
+        "arbitrum": "42161",
+        "gnosis": "100",
         "avalanche": "43114",
-        # "fantom": "250"
+        "fantom": "250"
     }
 
-    MODE = {
-        "slow": [10, 20, 30, 40, 50],  # <1min
-        "normal": [10, 30, 50, 70, 90],  # <30sec
-        "fast": [50, 60, 70, 80, 90],  # <10sec
-    }
+    # MODE = {
+    #     "slow": [10, 20, 30, 40, 50],  # <1min
+    #     "normal": [10, 30, 50, 70, 90],  # <30sec
+    #     "fast": [50, 60, 70, 80, 90],  # <10sec
+    # }
 
     abi = json.loads(pkg_resources.read_text(__package__, 'erc20.json'))['result']
 
-    def __init__(self, rpc_url, public_key, private_key, chain='ethereum'):
+    @staticmethod
+    def _get(url, params=None, headers=None):
+        """ Implements a get request """
+        try:
+            response = requests.get(url, params=params, headers=headers)
+            response.raise_for_status()
+            payload = response.json()
+        except requests.exceptions.ConnectionError as e:
+            print("ConnectionError when doing a GET request from {}".format(url))
+            payload = None
+        except requests.exceptions.HTTPError:
+            print("HTTPError {}".format(url))
+            payload = None
+        return payload
+
+    def __init__(self, rpc_url, public_key, private_key, chain='ethereum', broadcast_1inch=False):
         self.w3 = Web3(Web3.HTTPProvider(rpc_url))
         if chain == 'polygon' or chain == 'avalanche':
             self.w3.middleware_onion.inject(geth_poa_middleware, layer=0)
@@ -235,28 +241,29 @@ class TransactionHelper:
         self.private_key = private_key
         self.chain = chain
         self.chain_id = self.chains[chain]
+        self.broadcast_1inch = broadcast_1inch
 
-    def estimate_gas_fees(self, speed="fast", nb_blocks=3):
-        if speed not in self.MODE:
-            raise ValueError("Invalid speed")
+    # def estimate_gas_fees(self, speed="fast", nb_blocks=3):
+    #     if speed not in self.MODE:
+    #         raise ValueError("Invalid speed")
+    #
+    #     # baseFee: Set by blockchain, varies at each block, always burned
+    #     base_fee = self.w3.eth.get_block('pending').baseFeePerGas
+    #
+    #     # next baseFee: Overestimation of baseFee in next block, difference always refunded
+    #     next_base_fee = base_fee * 2
+    #
+    #     # priorityFee: Set by user, tip/reward paid directly to miners, never returned
+    #     reward_history = self.w3.eth.fee_history(
+    #         nb_blocks, 'pending', self.MODE[speed])['reward']
+    #     rewards = reduce(lambda x, y: x + y, reward_history)
+    #     avg_reward = sum(rewards) // len(rewards)
+    #
+    #     # Estimations: maxFee - (maxPriorityFee + baseFee actually paid) = Returned to used
+    #     return {"maxPriorityFeePerGas": avg_reward,
+    #             "maxFeePerGas": avg_reward + next_base_fee}
 
-        # baseFee: Set by blockchain, varies at each block, always burned
-        base_fee = self.w3.eth.get_block('pending').baseFeePerGas
-
-        # next baseFee: Overestimation of baseFee in next block, difference always refunded
-        next_base_fee = base_fee * 2
-
-        # priorityFee: Set by user, tip/reward paid directly to miners, never returned
-        reward_history = self.w3.eth.fee_history(
-            nb_blocks, 'pending', self.MODE[speed])['reward']
-        rewards = reduce(lambda x, y: x + y, reward_history)
-        avg_reward = sum(rewards) // len(rewards)
-
-        # Estimations: maxFee - (maxPriorityFee + baseFee actually paid) = Returned to used
-        return {"maxPriorityFeePerGas": avg_reward,
-                "maxFeePerGas": avg_reward + next_base_fee}
-
-    def build_tx(self, raw_tx, speed='fast', nb_blocks=3):
+    def build_tx(self, raw_tx, speed='high'):
         nonce = self.w3.eth.getTransactionCount(self.public_key)
         tx = raw_tx['tx']
         tx['nonce'] = nonce
@@ -264,15 +271,15 @@ class TransactionHelper:
         tx['chainId'] = int(self.chain_id)
         tx['value'] = int(tx['value'])
         tx['gas'] = int(tx['gas'] * 1.25)
-        if self.chain == 'ethereum' or self.chain == 'polygon' or self.chain == 'avalanche':
-            gas = self.estimate_gas_fees(speed, nb_blocks)
-            tx['maxPriorityFeePerGas'] = gas['maxPriorityFeePerGas']
-            tx['maxFeePerGas'] = gas['maxFeePerGas']
+        if self.chain == 'ethereum' or self.chain == 'polygon' or self.chain == 'avalanche' or self.chain == 'gnosis':
+            gas = self._get(self.gas_oracle+self.chain_id)
+            # print(gas)
+            # gas = requests.get(self.gas_oracle, params=self.chain_id)
+            tx['maxPriorityFeePerGas'] = int(gas[speed]['maxPriorityFeePerGas'])
+            tx['maxFeePerGas'] = int(gas[speed]['maxFeePerGas'])
             tx.pop('gasPrice')
-        elif self.chain == 'binance':
-            tx['gasPrice'] = int(tx['gasPrice'])
         else:
-            print('Chain Unsupported at this time')
+            tx['gasPrice'] = int(tx['gasPrice'])
         return tx
 
     def sign_tx(self, tx):
@@ -280,10 +287,19 @@ class TransactionHelper:
         return signed_tx
 
     def broadcast_tx(self, signed_tx, timeout=360):
-        tx_hash = self.w3.eth.send_raw_transaction(signed_tx.rawTransaction)
-        print(tx_hash.hex())
-        receipt = self.w3.eth.wait_for_transaction_receipt(tx_hash, timeout=timeout)
-        return receipt, tx_hash.hex()
+        if self.broadcast_1inch is True:
+            tx_json = signed_tx.rawTransaction
+            tx_json = {"rawTransaction": tx_json.hex()}
+            payload = requests.post('https://tx-gateway.1inch.io/v1.1/' + self.chain_id + '/broadcast', data=self.w3.toJSON(tx_json), headers={"accept": "application/json, text/plain, */*", "content-type": "application/json"})
+            tx_hash = json.loads(payload.text)
+            tx_hash = tx_hash['transactionHash']
+            receipt = self.w3.eth.wait_for_transaction_receipt(tx_hash, timeout=timeout)
+            return receipt, tx_hash
+        else:
+            tx_hash = self.w3.eth.send_raw_transaction(signed_tx.rawTransaction)
+            print(tx_hash.hex())
+            receipt = self.w3.eth.wait_for_transaction_receipt(tx_hash, timeout=timeout)
+            return receipt, tx_hash.hex()
 
     def get_ERC20_balance(self, contract_address, decimal=None):
         contract = self.w3.eth.contract(address=self.w3.toChecksumAddress(contract_address), abi=self.abi)
@@ -365,6 +381,5 @@ class OneInchOracle:
     #     return
 
 
-
 if __name__ == '__main__':
     pass
